var babylon = require('babylon');
var _ = require('lodash');
var escodegen = require('escodegen');

function getExportsNode(nodes) {
  return _.find(nodes, function(node) {
    var exp = node.expression;
    if (exp && exp.type === 'AssignmentExpression' && exp.left.type === 'MemberExpression' && exp.left.object.name === 'exports' && exp.right.type === 'ObjectExpression') {
      return exp.right;
    }
  })
}

function flat(property, parent, parentKey) {
  var oldKey = property.key.value;
  var newKey;
  if (_.contains(oldKey, '&')) {
    newKey = oldKey.split('&').join(parentKey);
  } else {
    newKey = parentKey + ' ' + oldKey;
  }
  property.key.value = newKey;
  parent.properties.push(property);
}

function flatten(object, parent, parentKey) {
  _.each(object.properties, function(p) {
    if (p.key.type === 'Literal' && _.startsWith(p.key.value, '.') && parent) {
      object.properties = _.without(object.properties, p);
      flat(p, parent, parentKey);
    }
    if (p.value.type === 'ObjectExpression') {
      flatten(p.value, object, p.key.value);
    }
  });
}

module.exports = function(content, map) {
  if (this.cacheable) {
    this.cacheable();
  }

  var tree = babylon.parse(content);
  var exportNode = getExportsNode(tree.body);

  flatten(exportNode.expression.right);
  return escodegen.generate(tree);
}
